"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NullLogger = exports.TrivialLogging = void 0;
const util_1 = require("util");
const path_1 = __importDefault(require("path"));
const pino_1 = __importDefault(require("pino"));
const nullLogger_1 = require("./lib/nullLogger");
Object.defineProperty(exports, "NullLogger", { enumerable: true, get: function () { return nullLogger_1.NullLogger; } });
let havePinoPretty = true;
try {
    require('pino-pretty');
}
catch (_) {
    havePinoPretty = false;
}
class TrivialLogging {
    constructor() {
        try {
            this.mainDir = path_1.default.dirname((require.main || {}).filename || '');
        }
        catch (err) {
            this.mainDir = process.cwd();
        }
        this._config = {
            nullLogger: !!process.env.LOG_NULL,
            options: {
                level: (process.env.LOG_LEVEL || 'debug').toLowerCase()
            }
        };
        this.root = this.setRootLogger();
    }
    get logger() { return this.root; }
    _modLogger(logger) {
        if (!logger.dump) {
            logger.dump = this.dump;
        }
        this._modLogFunc('warn', logger);
        this._modLogFunc('error', logger);
        return logger;
    }
    _modLogFunc(funcName, logger) {
        const origFunc = logger[funcName];
        logger[funcName] = function (...args) {
            args = args.map((arg) => {
                if (arg instanceof Error) {
                    if (arg.toJSON) {
                        return arg.toJSON();
                    }
                    else {
                        return {
                            code: arg.code,
                            message: arg.message,
                            stack: arg.stack
                        };
                    }
                }
                return arg;
            });
            origFunc.apply(this, args);
        };
    }
    init(config = { options: { level: 'debug' } }) {
        config.options = config.options || {};
        this._config = { ...config };
        this._config.nullLogger = !!process.env.LOG_NULL || config.nullLogger || false;
        this._config.options = { ...config.options };
        this._config.options.level = (process.env.LOG_LEVEL || config.level || (config.debug ? 'debug' : 'info')).toLowerCase();
        if (havePinoPretty) {
            this._config.options.prettyPrint = config.options.prettyPrint
                || (config.debug ? {
                    errorProps: '*',
                    levelFirst: false,
                    messageKey: 'msg',
                    timestampKey: 'time',
                    translateTime: 'h:MM:ss TT',
                    ignore: 'pid,hostname,moduleName'
                } : false);
        }
        this.setRootLogger();
    }
    setRootLogger(name = 'root', options) {
        this.root = this.getLogger(name, options);
        return this.root;
    }
    getLogger(name = 'logger', options) {
        options = { ...this._config.options, ...options, name };
        const logger = this._config.nullLogger ? new nullLogger_1.NullLogger() : pino_1.default(options);
        return this._modLogger(logger);
    }
    child(metadata = {}) {
        if (typeof metadata === 'string') {
            metadata = { metadata };
        }
        const logger = this.root.child(metadata);
        return this._modLogger(logger);
    }
    loggerFor(obj) {
        var _a;
        let filename;
        if (obj && typeof obj === 'object' && obj.constructor.name === 'Module') {
            filename = (_a = obj) === null || _a === void 0 ? void 0 : _a.filename;
        }
        else if (typeof obj === 'string') {
            filename = obj;
        }
        const moduleName = path_1.default.relative(this.mainDir, filename);
        return !this.root ? this.setRootLogger(moduleName) : this.root.child({ moduleName });
    }
    dump(obj, colors = true, depth = null, showHidden = false) {
        return util_1.inspect(obj, { colors, depth, showHidden });
    }
}
exports.TrivialLogging = TrivialLogging;
const logging = new TrivialLogging();
exports.default = logging;
module.exports = logging;
